@@ -4,12 +4,14 @@
 import os.path
 import warnings
 from pathlib import Path
-from typing import Optional, List, Iterable, Dict, Union, NamedTuple, Tuple
+from typing import Optional, List, Iterable, Dict, Union, Tuple
 
 import framefile
 
 from vtcff._args_subset import ArgsSubset
 from vtcff._codec import Codec
+from vtcff._codec_audio_copy import AudioCopy
+from vtcff._codec_video_copy import VideoCopy
 from vtcff._common import Scale
 from vtcff._filter_crop import Crop
 from vtcff._filter_pad import Pad
@@ -19,6 +21,14 @@ from vtcff._filter_zscale import ZscaleFilter, ColorSpaces
 from vtcff._time_span import BeginEndDuration
 
 
+class VideoCodecNotSpecifiedError(Exception):
+    pass
+
+
+class AudioCodecNotSpecifiedError(Exception):
+    pass
+
+
 def arg_i(path_or_pattern: Union[str, Path]) -> List[str]:
     path_or_pattern = str(path_or_pattern)
     if '*' in path_or_pattern:
@@ -36,12 +46,40 @@ def arg_i(path_or_pattern: Union[str, Path]) -> List[str]:
         return ["-i", path_or_pattern]
 
 
+def desynonimize(arg: str) -> str:
+    # https://superuser.com/q/835048
+    if arg in ('-vcodec', '-c:v'):
+        return '-codec:v'
+    if arg in ('-acodec', '-c:a'):
+        return '-codec:a'
+    return arg
+
+
+class CustomArgs:
+    def __init__(self):
+        self._before_i: ArgsSubset = ArgsSubset()
+        self._after_i: ArgsSubset = ArgsSubset()
+        self._video: ArgsSubset = ArgsSubset()
+        self._audio: ArgsSubset = ArgsSubset()
+
+    @property
+    def before_i(self) -> ArgsSubset:
+        return self._before_i
+
+    @property
+    def after_i(self) -> ArgsSubset:
+        return self._after_i
+
+    @property
+    def video(self) -> ArgsSubset:
+        return self._video
+
+    @property
+    def audio(self) -> ArgsSubset:
+        return self._audio
+
+
 class FfmpegCommand:
-    class CustomArgs(NamedTuple):
-        before_i: ArgsSubset = ArgsSubset()
-        after_i: ArgsSubset = ArgsSubset()
-        video: ArgsSubset = ArgsSubset()
-        audio: ArgsSubset = ArgsSubset()
 
     def __init__(self, use_zscale: bool = True):
 
@@ -71,7 +109,15 @@ class FfmpegCommand:
         self._dst_color_trc_meta: Optional[str] = None
         self._dst_colorspace_meta: Optional[str] = None
 
-        self.dst_codec_video: Optional[Codec] = None
+        # for vtcff we require video and audio codecs to be explicitly
+        # specified (although for ffmpeg the are optional). But the actual
+        # following filed can be set to None. Because user may want
+        # to specify codecs with .custom.video, for example.
+        # We will check the codecs are set at the very last stage: when
+        # we get all out arguments as strings.
+        self.dst_codec_video: Optional[Codec] = VideoCopy()
+        self.dst_codec_audio: Optional[Codec] = AudioCopy()
+
         self.dst_pixfmt: Optional[str] = None
 
         # про -color_range
@@ -85,7 +131,7 @@ class FfmpegCommand:
         # the FfmpegCommand object can generate parameters it understands.
         # But if you want to add some exotic args, this can be done by placing
         # then in fields of the following object
-        self.custom = FfmpegCommand.CustomArgs()
+        self.custom = CustomArgs()
 
         self.debug = False
 
@@ -341,7 +387,8 @@ class FfmpegCommand:
             # (https://stackoverflow.com/a/51224132)
             yield '-r', str(self.src_fps)
 
-    def _iter_known_all_after_i(self) -> Iterable[Union[str, Tuple[str, str]]]:
+    def _iter_known_all_after_i(self) -> Iterable[
+        Union[str, Tuple[str, Optional[str]]]]:
         if self.dst_time_range.begin:
             yield "-ss", str(self.dst_time_range.begin)
         if self.dst_time_range.duration is not None:
@@ -378,8 +425,12 @@ class FfmpegCommand:
         yield ('-sws_flags',
                'spline+accurate_rnd+full_chroma_int+full_chroma_inp')
 
+        if self.dst_codec_audio is not None:
+            for pair in self.dst_codec_audio.args():
+                yield pair
+
         if self.debug:
-            yield '-loglevel', 'debug'
+            yield '-loglevel', 'verbose'
 
         # последним в списке аргументов должно идти имя целевого файла.
         # Но здесь мы его не возвращаем - метод __iter__ добавит перед именем
@@ -398,7 +449,7 @@ class FfmpegCommand:
 
     def _iter_replacing_overrides(
             self,
-            args_or_pairs: Iterable[Union[str, Tuple[str, str]]],
+            args_or_pairs: Iterable[Union[str, Tuple[str, Optional[str]]]],
             overrides: Dict[str, Optional[str]]) \
             -> Iterable[str]:
 
@@ -420,7 +471,8 @@ class FfmpegCommand:
                         yield other_v
                 else:
                     yield k
-                    yield v
+                    if v is not None:
+                        yield v
                 returned_keys.add(k)
             else:
                 raise TypeError
@@ -436,7 +488,7 @@ class FfmpegCommand:
                 if v is not None:
                     yield v
 
-    def __iter__(self) -> Iterable[str]:
+    def _iter_almost_final(self) -> Iterable[str]:
         """Возвращает аргументы к команде ffmpeg списком."""
 
         for x in self._iter_replacing_overrides(
@@ -464,5 +516,17 @@ class FfmpegCommand:
             raise ValueError("Output file not specified")
         yield str(self.dst_file)
 
+    def __iter__(self) -> Iterable[str]:
+        args = list(desynonimize(arg) for arg in self._iter_almost_final())
+
+        if not any(s in args for s in ("-codec:v", "-vn")):
+            raise VideoCodecNotSpecifiedError(args)
+
+        # The -vn / -an / -sn / -dn options can be used to skip inclusion
+        # of video, audio, subtitle and data streams respectively
+        if not any(s in args for s in ("-codec:a", "-an")):
+            raise AudioCodecNotSpecifiedError
+        return iter(args)
+
     def __str__(self):
         return ' '.join(iter(self))
